/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

#ifndef	__CEBOMSR_API_CPP_H__
#define	__CEBOMSR_API_CPP_H__

#ifdef _MSC_VER
	typedef __int32 int32_t;
	typedef unsigned __int32 uint32_t;
	typedef __int64 int64_t;
	typedef unsigned __int64 uint64_t;
#else
#	include <stdint.h>
#endif

#include <string>
#include <vector>
#include <map>
#include <cassert>
#include <algorithm>

// To prevent naming conflicts, move to different name space.
namespace _c {
	#include <cebomsr.h>
}

namespace CeboMsr {

class Device;

/**
 * Values to be used in enumeration.
 */
class DeviceType {
	friend class LibraryInterface;
	_c::CeboMsrType _msrType;
	std::string _name;
	DeviceType(_c::CeboMsrType msrType, const std::string &name) : _msrType(msrType), _name(name) {}
	_c::CeboMsrType getInterfaceId() const { return _msrType; }
	static DeviceType &getTypeById(_c::CeboMsrType msrType);
public:

	/**
	 * Return instance name, "Usb", "All", ...
	 */
	std::string getName() const { return _name; }

	bool operator==(const DeviceType &o) const {
		return _msrType == o._msrType;
	}

	static DeviceType All;
	static DeviceType Usb;
	static DeviceType CeboLC;
	static DeviceType CeboStick;
	static DeviceType CeboDFN;
};

/**
 * Base class for any object that can act as input.
 */
class Input {
	friend class Device;
	friend class AnalogInput;
	friend class DigitalPort;
	friend class Counter;
	friend class Trigger;
	friend struct input2io;

	_c::CeboMsrIo _inputIo;
	_c::CeboMsrIo getInputIo() const { return _inputIo; }
protected:
	explicit Input(const _c::CeboMsrIo &io) : _inputIo(io) {}
public:
	virtual ~Input() {}
};

/**
 * Base class for any object that can act as output.
 */
class Output {
	friend class Device;
	friend class AnalogOutput;
	friend class DigitalPort;
	friend struct output2io;

	_c::CeboMsrIo _outputIo;
	_c::CeboMsrIo getOutputIo() const { return _outputIo; }
protected:
	explicit Output(const _c::CeboMsrIo &io) : _outputIo(io) {}
public:
	virtual ~Output() {}
};

/**
 * API internal helper.
 */
struct input2io {
	_c::CeboMsrIo operator()(const Input& io) const { return io._inputIo; }
};

/**
 * API internal helper.
 */
struct output2io {
	_c::CeboMsrIo operator()(const Output& io) const { return io._outputIo; }
};

/**
 * These objects are used to set the range of analog inputs and outputs.
 * Direct construction is not possible, getSupportedRanges() of AnalogInput and
 * AnalogOutput can be used to request the possible ranges of an input or output.
 */
class Range {
	friend class Device;
	friend class AnalogInput;
	friend class AnalogOutput;

	_c::CeboMsrRange _range;
	float _minValue, _maxValue;
	int _defaultInterChannelDelay;
	Range(const _c::CeboMsrRange &range, float minValue, float maxValue, int defaultInterChannelDelay) :
		_range(range), _minValue(minValue), _maxValue(maxValue), _defaultInterChannelDelay(defaultInterChannelDelay) {}

public:

	/**
	 * Return lower voltage.
	 */
	float getMinValue() const { return _minValue; }

	/**
	 * Return upper voltage.
	 */
	float getMaxValue() const { return _maxValue; }

	bool operator==(const Range &range) const {
		return _range == range._range;
	}
};

/**
 * Offers functionality of the different kind of analog inputs.
 * Direct construction is not possible, instances can be retrieved calling
 * getSingleEndedInputs() or getDifferentialInputs() of the respective instance
 * of class Device.
 */
class AnalogInput : public Input {
	friend class Device;
	friend class InputFrame;

	enum Style { SingleEnded, Differential };

	_c::CeboMsrHandle _handle;
	Style _style;
	int _styleIndex;
	std::vector<Range> _supportedRanges;
	int _minInterChannelDelay;
	std::string _name;

	AnalogInput(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, Style style, int styleIndex);

public:

	/**
	 * Return list of the supported ranges.
	 */
	const std::vector<Range> &getSupportedRanges() const { return _supportedRanges; }

	/**
	 * Request default interchannel delay at the given range setting.
	 * @param range Range for which this value is requested.
	 */
	int getDefaultInterChannelDelay(const Range &range) const;

	/**
	 * Return minimum interchannel delay.
	 */
	int getMinInterChannelDelay() const { return _minInterChannelDelay; }

	/**
	 * Set new range, interchannel delay is set to the default for this range.
	 * @param range Range level to set.
	 */
	void setParameters(const Range &range) const;

	/**
	 * Set range and user specified interchannel delay.
	 * @param range Range level to set.
	 * @param interChannelDelay Interchannel delay in microseconds.
	 * @return Corrected interchannel delay in microseconds.
	 */
	int setParameters(const Range &range, int interChannelDelay) const;

	/**
	 * Read active range level.
	 */
	Range getRange() const;

	/**
	 * Read active interchannel delay.
	 */
	int getInterChannelDelay() const;

	/**
	 * Read current voltage value.
	 */
	float read() const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * Offers functionality for analog outputs. Direct construction
 * is not possible. Instances can be retrieved calling getAnalogOutputs()
 * of the respective Device class.
 */
class AnalogOutput : public Output {
	friend class Device;
	friend class OutputFrame;

	_c::CeboMsrHandle _handle;
	int _index;
	std::vector<Range> _supportedRanges;
	std::string _name;

	AnalogOutput(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index);

public:

	/**
	 * Return list of the supported ranges.
	 */
	const std::vector<Range> &getSupportedRanges() const { return _supportedRanges; }

	/**
	 * Set new range.
	 */
	void setParameters(const Range &range) const;

	/**
	 * Read active range level.
	 */
	Range getRange() const;

	/**
	 * Set voltage level directly.
	 * @param value Value in volt to set.
	 */
	void write(float value) const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * Offers functionality to work with digital ports. Direct construction
 * is not possible. Instances can be retrieved calling getDigitalPorts() of
 * the respective Device instance.
 */
class DigitalPort : public Input, public Output {
	friend class Device;
	friend class InputFrame;
	friend class OutputFrame;

	_c::CeboMsrHandle _handle;
	int _index;
	int _ioCount;
	std::string _name;
	std::vector<std::string> _ioNames;

	DigitalPort(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index);

public:

	/**
	 * Set output mask. The default mask is 0.
	 * @param mask Mask to use, bits that have state '1' specify output direction.
	 */
	void setOutputEnableMask(int mask) const;

	/**
	 * Return count of I/O's for the specified port.
	 */
	int getIoCount() const { return _ioCount; }

	/**
	 * Read value currently active on the port.
	 */
	int read() const;

	/**
	 * Set output I/O's. Only the bits that have been defined as output 
	 * using setOutputEnableMask() are modified.
	 * @param value Value to set on this port. 
	 */
	void write(int value) const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }

	/**
	 * Return name of the I/O as specified by io.
	 * @param io I/O index in range (0 <= io < getIoCount()).
	 */
	std::string getIoName(int io) const { return _ioNames.at(io); }
};

/**
 * Offers functionality for counter periphery. Instances can be retrieved calling
 * getCounters() of the respective Device instance.
 */
class Counter : public Input {
public:
	// The possible counter configuration values.
	enum CounterConfig {
		RisingEdge = _c::CeboMsrCounterConfigRisingEdge,
		FallingEdge = _c::CeboMsrCounterConfigFallingEdge,
		Alternating = _c::CeboMsrCounterConfigAlternating
	};

	friend class Device;
	friend class InputFrame;

	_c::CeboMsrHandle _handle;
	int _index;
	std::string _name;

	Counter(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index);

public:

	/**
	 * Reset counter to 0.
	 */
	void reset() const;

	/**
	 * Enable or disable counter.
	 * @param enable State to set.
	 */
	void setEnabled(bool enable) const;

	/**
	 * Get current counter state.
	 */
	bool isEnabled() const;

	/**
	 * Define behavior of the counter.
	 * @param triggerConfig Use one of the values from Counter::CounterConfig.
	 */
	void setConfig(CounterConfig counterConfig) const;

	/**
	 * Request configuration of the counter.
	 * @return One of the values from Counter::CounterConfig.
	 */
	CounterConfig getConfig() const;

	/**
	 * Read current counter value.
	 */
	int64_t read() const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * Offers functionality for triggers. Instances can be retrieved calling getTriggers()
 * of the respective Device instance.
 */
class Trigger : public Input {
public:
	// The possible trigger configuration values.
	enum TriggerConfig {
		OutputPulse = _c::CeboMsrTriggerOutputPulse,
		OutputAlternating = _c::CeboMsrTriggerOutputAlternating,
		InputRisingEdge = _c::CeboMsrTriggerInputRisingEdge,
		InputFallingEdge = _c::CeboMsrTriggerInputFallingEdge,
		InputAlternating = _c::CeboMsrTriggerInputAlternating
	};

private:

	friend class Device;
	friend class InputFrame;

	_c::CeboMsrHandle _handle;
	int _index;
	std::string _name;

	Trigger(_c::CeboMsrHandle handle, const _c::CeboMsrIo &io, int index);

public:

	/**
	 * Enable or disable trigger.
	 * @param enable State to set.
	 */
	void setEnabled(bool enable) const;

	/**
	 * Get current trigger state.
	 */
	bool isEnabled() const;

	/**
	 * Define behavior of the trigger.
	 * @param triggerConfig Use one of the values from Trigger::TriggerConfig.
	 */
	void setConfig(TriggerConfig triggerConfig) const;

	/**
	 * Request configuration of the trigger.
	 * @return One of the values from Trigger::TriggerConfig.
	 */
	TriggerConfig getConfig() const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * Offers functionality of for LED's. Instances can be retrieved using getLeds()
 * of the respective Device instance.
 */
class Led {
	friend class Device;

	_c::CeboMsrHandle _handle;
	_c::CeboMsrLed _led;
	int _index;
	std::string _name;

	Led(_c::CeboMsrHandle handle, const _c::CeboMsrLed &led, int index);

public:

	/**
	 * Set state of the on-board led.
	 * @param enable State to switch the indicator to.
	 */
	void setEnabled(bool enable) const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * Offers functionality for current sources. Instances can be retrieved using
 * getCurrentSources() of the respective Device instance.
 */
class CurrentSource {
	friend class Device;

	_c::CeboMsrHandle _handle;
	_c::CeboMsrCurrentSource _source;
	int _index;
	std::string _name;

	CurrentSource(_c::CeboMsrHandle handle, const _c::CeboMsrCurrentSource &source, int index);

public:

	/**
	 * Returns the real current of the constant current source in micro ampere.
	 */
	float getReferenceCurrent() const;

	/**
	 * Returns name of the component.
	 */
	std::string getName() const { return _name; }
};

/**
 * One or more instances of this interface are returned when reading one or multiple
 * frames previously setup using Device::setupInputFrame(). It allows
 * simple access to all captured data.
 */
class InputFrame {
	friend class Device;

	std::vector<float> _singleEnded, _differential;
	std::vector<int> _digital;
	std::vector<unsigned int> _counter;
	std::vector<int> _trigger;

	template<typename V, typename I>
	InputFrame(
			V sebegin, V seend,
			V diffbegin, V diffend,
			I digbegin, I digend,
			I cntbegin, I cntend,
			I trgbegin, I trgend) :
		_singleEnded(sebegin, seend),
		_differential(diffbegin, diffend),
		_digital(digbegin, digend),
		_counter(cntbegin, cntend),
		_trigger(trgbegin, trgend)
	{}

public:

	/**
	 * Read value from single ended input with given index.
	 * @param index Zero based index of single ended input in question.
	 * @return Value of this input in voltage.
	 */
	float getSingleEnded(int index) const {
		assert(index >= 0 && index < (int)_singleEnded.size() && "Index out of range.");
		return _singleEnded[index];
	}

	/**
	 * Read value from differential input with given index.
	 * @param index Zero based index of differential input in question.
	 * @return Value of this input in voltage.
	 */
	float getDifferential(int index) const {
		assert(index >= 0 && index < ((int)_differential.size()) && "Index out of range.");
		return _differential[index];
	}

	/**
	 * Read value from digital port with given index.
	 * @param index Zero based index of digital port in question.
	 * @return Active mask of digital port.
	 */
	int getDigitalPort(int index) const {
		assert(index >= 0 && index < (int)_digital.size() && "Index out of range.");
		return _digital[index];
	}

	/**
	 * Read level from trigger with given index.
	 * @param index Zero based index of trigger in question.
	 * @return Current level of trigger, true equals a high level.
	 */
	bool getTrigger(int index) const {
		assert(index >= 0 && index < (int)_trigger.size() && "Index out of range.");
		return 0 != _trigger[index];
	}

	/**
	 * Read value of specified counter.
	 * @param index Zero based index of trigger in question.
	 * @return Current counter value.
	 */
	int64_t getCounter(int index) const {
		assert(index >= 0 && index < (int)_counter.size() && "Index out of range.");
		return _counter[index];
	}
};

/**
 * Class to use when setting multiple outputs simultaneous. Direct construction is prohibited,
 * as the count of the the various elements varies from device type to device type. Instances
 * must be generated calling Device::createOutputFrame().
 */
class OutputFrame {
	friend class Device;

	std::vector<float> _analog;
	std::vector<int> _digital;

	OutputFrame(int anc, int dic) : _analog(anc, 0.f), _digital(dic, 0) {}

public:

	/**
	 * Define value that should be set on the specified digital port.
	 * @param index Zero based index of port to specify.
	 * @param value Value to set at this port.
	 */
	void setDigitalPort(int index, int value) {
		assert(index >= 0 && index < (int)_digital.size() && "Index out of range.");
		_digital[index] = value;
	}

	/**
	 * Define value that should be set on the specified analog output.
	 * @param index Zero based index of output to specify.
	 * @param value Value to set at this output.
	 */
	void setAnalogOutput(int index, float value) {
		assert(index >= 0 && index < (int)_analog.size() && "Index out of range.");
		_analog[index] = value;
	}
};

typedef std::vector<Input> InputVector;
typedef std::vector<Output> OutputVector;
typedef std::vector<Range> RangeVector;
typedef std::vector<AnalogInput> AnalogInputVector;
typedef std::vector<AnalogOutput> AnalogOutputVector;
typedef std::vector<DigitalPort> DigitalPortVector;
typedef std::vector<Counter> CounterVector;
typedef std::vector<Trigger> TriggerVector;
typedef std::vector<Led> LedVector;
typedef std::vector<CurrentSource> CurrentSourceVector;
typedef std::vector<InputFrame> InputFrameVector;
typedef std::vector<OutputFrame> OutputFrameVector;

/**
 * Allows to use the streaming operator to fill the vector.
 */
inline InputVector &operator<<(InputVector &v, const Input &i) {
	v.push_back(i);
	return v;
}

/**
 * Allows to use the streaming operator to fill the vector.
 */
inline OutputVector &operator<<(OutputVector &v, const Output &i) {
	v.push_back(i);
	return v;
}

/**
 * The primary object to communicate with a CeboMsr-compatible device. Every instance holds a single
 * connection to a device. Instances are created during enumeration in LibraryInterface::enumerate().
 */
class Device {
	friend class AnalogInput;
	friend class AnalogOutput;
	friend class DigitalPort;
	friend class Counter;
	friend class Trigger;
	friend class Led;
	friend class CurrentSource;
	friend class LibraryInterface;

	std::string _ident, _firmwareVersion;
	DeviceType _type;
	_c::CeboMsrHandle _handle;
	int _maxInputFrames;

	AnalogInputVector _singleEnded;
	AnalogInputVector _differential;
	AnalogOutputVector _analogOutputs;
	DigitalPortVector _digitalPorts;
	CounterVector _counters;
	TriggerVector _triggers;
	LedVector _leds;
	CurrentSourceVector _currentSources;

	int getInputFrameFloatCount() const { return (int)_singleEnded.size() + (int)_differential.size(); }
	int getInputFrameIntCount() const { return (int)_digitalPorts.size() + (int)_counters.size() + (int)_triggers.size(); }
	int getOutputFrameFloatCount() const { return (int)_analogOutputs.size(); }
	int getOutputFrameIntCount() const { return (int)_digitalPorts.size(); }
	int getMaxInputFramesCount() const { return _maxInputFrames; }

	// No direct construction, this is done by the enumerator.
	Device(const std::string &ident, DeviceType type, const std::string &firmwareVersion);

public:

	/**
	 * Default constructor.
	 */
	Device() : _firmwareVersion(""), _type(DeviceType::All), _handle(-1), _maxInputFrames(-1) {}

	/**
	 * Open device.
	 */
	void open();

	/**
	 * Close device.
	 */
	void close();

	/**
	 * Restore startup defaults in device.
	 */
	void resetDevice();

	/**
	 * Reset specified elements in device.
	 * @param mask Bitvector constructed of Device::FlagReset*.
	 */
	void resetPeripherals(int mask);

	/**
	 * Return unique id of device.
	 */
	std::string getIdentifier() const { return _ident; }

	/**
	 * Get actual type of device.
	 */
	DeviceType getDeviceType() const { return _type; }

	/**
	 * Read string which contains the firmware version.
	 */
	std::string getFirmwareVersion() const { return _firmwareVersion; }

	/**
	 * Return serial number of device.
	 */
	std::string getSerialNumber() const;

	/**
	 * Read device temperature in degree celsius.
	 */
	float getTemperature() const;

	/**
	 * Calculate maximum count of frames that can be buffered on the device when using 
	 * multiple frame mode. Depends on the actively selected inputs.
	 * @return Frame count.
	 */
	int calculateMaxBufferedInputFrames() const;

	/**
	 * Define the inputs that were included in the next frame request 
	 * (readFrame(), readBlocking() and readNonBlocking()).
	 * @param inputs Collection of inputs.
	 */
	void setupInputFrame(const InputVector &inputs);

	/**
	 * Initialize a buffered data acquisition. Frames are buffered in the device memory, 
	 * so frame loss is not possible.
	 * @param frameRate Frequency in Hz to capture frame.
	 * @param frameCount Maximum count of frames to capture. Maximum possible frames can 
	 *                   calculated using calculateMaxBufferedInputFrames().
	 * @param externalStarted If true, sampling starts if an external trigger is detected, 
	 *                        immediately otherwise.
	 * @return Corrected frame rate.
	 */
	float startBufferedDataAcquisition(float frameRate, int frameCount, bool externalStarted);

	/**
	 * Initialize a buffered data acquisition. Frames are buffered in the device memory, 
	 * so frame loss is not possible. For every detected trigger, one frame is captured. 
	 * Too high frequency on the external trigger will create frame loss.
	 * @param frameCount Maximum count of frames to capture. Maximum possible frames can calculated 
	 *                   using calculateMaxBufferedInputFrames().
	 */
	void startBufferedExternalTimedDataAcquisition(int frameCount);

	/**
	 * Initialize a continuous data acquisition. Data must be read as fast as possible using 
	 * readBlocking()/readNonBlocking(), otherwise  an device buffer overflow may occur.
	 * @param frameRate Frequency in Hz to capture frame.
	 * @param externalStarted If true, sampling starts if an external trigger is detected, 
	 *                        immediately otherwise.
	 * @return Corrected frame rate.
	 */
	float startContinuousDataAcquisition(float frameRate, bool externalStarted);

	/**
	 * Initialize a continuous data acquisition. Data must be read as fast as possible 
	 * using readBlocking()/readNonBlocking(), otherwise an device buffer overflow may occur. 
	 * For every detected trigger, one frame is captured. Too high frequency on the external
	 * trigger will create frame loss.
	 */
	void startContinuousExternalTimedDataAcquisition();

	/**
	 * Stop any form of hardware managed data acquisition.
	 */
	void stopDataAcquisition();

	/**
	 * Read exact frameCount frames, return after completion, blocks until all data is available.
	 * @param frameCount Count of frames to read.
	 * @return Collection of read frames.
	 */
	InputFrameVector readBlocking(int frameCount);

	/**
	 * Read at much possible available frames, return immediately.
	 * @return Collection of frames available at the moment of reading, 0 <= count <= max possible.
	 */
	InputFrameVector readNonBlocking();

	/**
	 * Read multiple inputs using the previously defined list (setupInputFrame()).
	 * @return Frame that contains the values of all specified inputs.
	 */
	InputFrame readFrame();

	/**
	 * Define the outputs that should be set using the next frame based out transfer.
	 * @param outputs Collection of outputs.
	 */
	void setupOutputFrame(const OutputVector &outputs);

	/**
	 * Write all output frames, return after completion, blocks until all data is sent.
	 * @param frames Frames to write.
	 */
	void writeBlocking(const OutputFrameVector &frames);

	/**
	 * Write as much output frames as possible. Return the frames that remains unsent.
	 * @param frames Frames to write.
	 * @return Frames not sent.
	 */
	OutputFrameVector writeNonBlocking(const OutputFrameVector &frames);

	/**
	 * Write multiple outputs at once. All outputs specified using setupOutputFrame() are modified using
	 * the values in the frame.
	 * @param frame Instance that holds all values to set.
	 */
	void writeFrame(const OutputFrame &frame);

	/**
	 * Create OutputFrame instance specific to this device.
	 * @return New instance of OutputFrame.
	 */
	OutputFrame createOutputFrame() const { 
		return OutputFrame(getOutputFrameFloatCount(), getOutputFrameIntCount()); 
	}

	/**
	 * Create the given count of output frames.
	 * @param count Count of frames to generate.
	 */
	OutputFrameVector createOutputFrames(int count) const;

	/**
	 * Return list of single ended analog inputs.
	 */
	const AnalogInputVector &getSingleEndedInputs() const { return _singleEnded; }

	/**
	 * Return list of differential analog inputs.
	 */
	const AnalogInputVector &getDifferentialInputs() const { return _differential; }

	/**
	 * Return list of analog outputs.
	 */
	const AnalogOutputVector &getAnalogOutputs() const { return _analogOutputs; }

	/**
	 * Return list of digital ports.
	 */
	const DigitalPortVector &getDigitalPorts() const { return _digitalPorts; }

	/**
	 * Return list of counters.
	 */
	const CounterVector &getCounters() const { return _counters; }

	/**
	 * Return list of triggers.
	 */
	const TriggerVector &getTriggers() const { return _triggers; }

	/**
	 * Return list of current sources.
	 */
	const CurrentSourceVector &getCurrentSources() const { return _currentSources; }

	/**
	 * Return list of LED's.
	 */
	const LedVector &getLeds() const { return _leds; }

	/**
	 * Valid flags to use with Device::resetPeripherals().
	 */
	static const int FlagResetInputFifo = _c::CeboMsrPeripheralResetMaskInputFifo;
};

typedef std::vector<Device> DeviceVector;

/**
 * Class that is responsible for all device independent functionality.
 */
class LibraryInterface {

	// Prevent construction.
	LibraryInterface();

	// Prevent copy and assignment.
	explicit LibraryInterface(const LibraryInterface &);
	LibraryInterface &operator=(LibraryInterface&);

public:

	/**
	 * Request version of the CeboMsr-Base-API.
	 */
	static std::string getApiVersion();

	/**
	 * Enumerate devices on this system. Already opened devices are not included.
	 */
	static DeviceVector enumerate(const DeviceType &type);

	/// *** unofficial experimental feature, don't use ***
	static DeviceVector enumerateNetwork(const char *server, int port, const DeviceType &type);
};

} // namespace CeboMsr

#endif // __CEBOMSR_API_CPP_H__
