/* 
  CESYS Software License - Version 1.0 - January 1st, 2016

  Permission is hereby granted, free of charge, to any person or organization
  obtaining a copy of the software and accompanying documentation covered by
  this license (the "Software") to use, reproduce, display, distribute,
  execute, and transmit the Software, and to prepare derivative works of the
  Software, and to permit third-parties to whom the Software is furnished to
  do so, all subject to the following:

  The software must only be used to operate hardware manufactured by 
  CESYS GmbH, Herzogenaurach.

  The copyright notices in the Software and this entire statement, including
  the above license grant, this restriction and the following disclaimer,
  must be included in all copies of the Software, in whole or in part, and
  all derivative works of the Software, unless such copies or derivative
  works are solely in the form of machine-executable object code generated by
  a source language processor.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
  SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
  FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.
  
  Written by Thomas Hoppe <thomas.hoppe@cesys.com>, 2016
*/  

package com.cesys.cebo.cebomsr;

/**
 * One or more instances of this interface are returned when reading one or multiple
 * frames previously setup using {@link Device}.setupInputFrame(). It allows
 * simple access to all captured data.
 *
 * @author Thomas Hoppe
 */
public final class InputFrame {
	private final float _analogValues[];
	private final int _digitalValues[];
	private final int _singleEndedOffset, _singleEndedCount, _differentialOffset, _differentialCount;
	private final int _digitalPortOffset, _digitalPortCount, _counterOffset, _counterCount, _triggerOffset, _triggerCount;

	InputFrame(
			float[] analogValues,
			int[] digitalValues,
			int singleEndedOffset, int singleEndedCount,
			int differentialOffset, int differentialCount,
			int digitalPortOffset, int digitalPortCount,
			int counterOffset, int counterCount,
			int triggerOffset, int triggerCount) {
		_digitalValues = digitalValues;
		_analogValues = analogValues;
		_singleEndedOffset = singleEndedOffset;
		_singleEndedCount = singleEndedCount;
		_differentialOffset = differentialOffset;
		_differentialCount = differentialCount;
		_digitalPortOffset = digitalPortOffset;
		_digitalPortCount = digitalPortCount;
		_counterOffset = counterOffset;
		_counterCount = counterCount;
		_triggerOffset = triggerOffset;
		_triggerCount = triggerCount;
	}

	/**
	 * Read value from single ended input with given index.
	 * @param index Zero based index of single ended input in question.
	 * @return Value of this input in voltage.
	 */
	public float getSingleEnded(int index) {
		// Must be checked, as the arrays can span over multiple frames.
		if (index < 0 || index >= _singleEndedCount)
			throw new IndexOutOfBoundsException();
		return _analogValues[_singleEndedOffset + index];
	}

	/**
	 * Read value from differential input with given index.
	 * @param index Zero based index of differential input in question.
	 * @return Value of this input in voltage.
	 */
	public float getDifferential(int index) {
		// Must be checked, as the arrays can span over multiple frames.
		if (index < 0 || index >= _differentialCount)
			throw new IndexOutOfBoundsException();
		return _analogValues[_differentialOffset + index];
	}

	/**
	 * Read value from digital port with given index.
	 * @param index Zero based index of digital port in question.
	 * @return Active mask of digital port.
	 */
	public int getDigitalPort(int index) {
		// Must be checked, as the arrays can span over multiple frames.
		if (index < 0 || index >= _digitalPortCount)
			throw new IndexOutOfBoundsException();
		return _digitalValues[_digitalPortOffset + index];
	}

	/**
	 * Read level from trigger with given index.
	 * @param index Zero based index of trigger in question.
	 * @return Current level of trigger, true equals a high level.
	 */
	public boolean getTrigger(int index) {
		if (index < 0 || index >= _triggerCount)
			throw new IndexOutOfBoundsException();
		return 0 != _digitalValues[_triggerOffset + index];
	}

	/**
	 * Read value of specified counter.
	 * @param index Zero based index of counter in question.
	 * @return Current counter value.
	 */
	public long getCounter(int index) {
		if (index < 0 || index >= _counterCount)
			throw new IndexOutOfBoundsException();
		return _digitalValues[_counterOffset + index] & 0xffffffffL;
	}

	public String toString() {
		StringBuilder sb = new StringBuilder();
		sb.append("InputFrame: \n");
		for (int i = 0; i < _singleEndedCount; ++i)
			sb.append(String.format("SingleEnded #%d = %.2f V \n", i, getSingleEnded(i)));
		for (int i = 0; i < _differentialCount; ++i)
			sb.append(String.format("Differential #%d = %.2f V \n", i, getDifferential(i)));
		for (int i = 0; i < _digitalPortCount; ++i)
			sb.append(String.format("DigitalPort #%d = %d (0x%02x) \n", i, getDigitalPort(i), getDigitalPort(i)));
		for (int i = 0; i < _counterCount; ++i)
			sb.append(String.format("Counter #%d = %d (0x%02x) \n", i, getCounter(i), getCounter(i)));
		for (int i = 0; i < _triggerCount; ++i)
			sb.append(String.format("Trigger #%d = %s \n", i, getTrigger(i) ? "high" : "low"));
		return sb.toString();
	}
}
